% zoom plotter to plot the motion of a 4-group lens through zoom
% First and last groups are fixed, middle two groups are moving

%% Input parameters
EFLshort = 12.25; % the short focal length of the zoom range
EFLlong = 196; % the long focal length of the zoom range
% These parameters can be specified here, or they can be taken as random values at a later part of the code
BFL = 30; % the back focal length
OAL = 220; % the overall length from the first group to the last group
TotalTrack = 250; % the total track from first group to image plane

%% Search over group focal lengths
% Specify design form (sign of power in each group) and limits on focal length amplitudes
EFLlowerlimit = 10;
EFLupperlimit = 300;
designform = [-1 1 1 1];
f1min = designform(1)*EFLlowerlimit;
f1max = designform(1)*EFLupperlimit;
f2min = designform(2)*EFLlowerlimit;
f2max = designform(2)*EFLupperlimit;
f3min = designform(3)*EFLlowerlimit;
f3max = designform(3)*EFLupperlimit;
f4min = designform(4)*EFLlowerlimit;
f4max = designform(4)*EFLupperlimit;

% % Or specify any limits on focal length range
% f1min = 1;
% f1max = 10000;
% f2min = 1;
% f2max = 10000;
% f3min = -1000;
% f3max = -1;
% f4min = -1000;
% f4max = -1;

%% Search over BFL and OAL, specify range of values to look at
% Limits for random BFL
BFLmin = 20;
BFLmax = 150;
% Limits for random OAL
OALmin = 0;
OALmax = 300;

%% Calculate zoom motion for a random collection of focal lengths and see if it causes crashing
EFL = linspace(EFLshort,EFLlong,51);
f1out = [];f2out = [];f3out = [];f4out = [];root = [];BFLout = [];OALout = [];

% Specify a number of random trials - try a small number at first (50 or
% 100) and increase it until you get the desired number of output solutions
ntrials = 100000;

pause off
for ij = 1:ntrials
% Generate group focal lengths randomly
f1 = f1min + (f1max-f1min)*rand(1);
f2 = f2min + (f2max-f2min)*rand(1);
f3 = f3min + (f3max-f3min)*rand(1);
f4 = f4min + (f4max-f4min)*rand(1);

% Generate BFL randomly
BFL = BFLmin + (BFLmax-BFLmin)*rand(1);

% % Generate OAL randomly
% OAL = OALmin + (OALmax-OALmin)*rand(1);

% Or constrain OAL to maintain constant total track
OAL = TotalTrack - BFL;

L = OAL - f1 - f4*BFL/(BFL-f4);
M = -EFL.*(OAL-L-f1)./(BFL*f1);
c = L*(f2+f3)+((M-1).^2).*f2*f3./M;

if L^2-4*1*min(c) >=0 && L^2-4*1*max(c) >= 0

% Check positive root first; a = 1;b = -L;
t2 = (L+sqrt(L^2-4*1*c))./(2*1);
x = L+((M-1).*t2+L)./((M-1)-M.*t2/f2);
t1 = OAL-x-f4*BFL/(BFL-f4);
t3 = OAL-t1-t2;
lens4z = BFL;
lens3z = t3+lens4z;
lens2z = t2+t3+lens4z;
lens1z = t1+t2+t3+lens4z;

% check for valid solutions (lens1z > lens2z > lens3z > lens4z)
if sum(lens2z > lens1z) + sum(lens3z > lens2z) + sum(lens4z > lens3z) > 0
    % There are no valid solutions
else
    f1out(end+1) = f1;f2out(end+1) = f2;f3out(end+1) = f3;f4out(end+1) = f4;
    BFLout(end+1) = BFL;
    OALout(end+1) = OAL;
    root(end+1) = 1;
%     figure;plot(EFL,lens1z,'b',EFL,lens2z,'r',EFL,lens3z,'g',EFL,lens4z,'m')
%     xlabel('EFL (mm)');ylabel('Distance from Image (mm)');legend('Lens1','Lens2','Lens3','Lens4')
%     title(['Positive Root: f_1 = ' num2str(f1) '; f_2 = ' num2str(f2) '; f_3 = ' num2str(f3) '; f_4 = ' num2str(f4)])
    pause
end

% check negative root
t2 = (L-sqrt(L^2-4*1*c))./(2*1);
x = L+((M-1).*t2+L)./((M-1)-M.*t2/f2);
t1 = OAL-x-f4*BFL/(BFL-f4);
t3 = OAL-t1-t2;
lens4z = BFL;
lens3z = t3+lens4z;
lens2z = t2+t3+lens4z;
lens1z = t1+t2+t3+lens4z;

% check for valid solutions (lens1z > lens2z > lens3z > lens4z)
if sum(lens2z > lens1z) + sum(lens3z > lens2z) + sum(lens4z > lens3z) > 0
    % There are no valid solutions
else
    f1out(end+1) = f1;f2out(end+1) = f2;f3out(end+1) = f3;f4out(end+1) = f4;
    BFLout(end+1) = BFL;
    OALout(end+1) = OAL;
    root(end+1) = -1;
%     figure;plot(EFL,lens1z,'b',EFL,lens2z,'r',EFL,lens3z,'g',EFL,lens4z,'m')
%     xlabel('EFL (mm)');ylabel('Distance from Image (mm)');legend('Lens1','Lens2','Lens3','Lens4')
%     title(['Negative Root: f_1 = ' num2str(f1) '; f_2 = ' num2str(f2) '; f_3 = ' num2str(f3) '; f_4 = ' num2str(f4)])
    pause
end
end

end

if length(f1out) > 0
TTout = OALout+BFLout;
EFLRSS = f1out.^2+f2out.^2+f3out.^2+f4out.^2;
PTB = 1./f1out+1./f2out+1./f3out+1./f4out;
minEFL = min(abs([f1out;f2out;f3out;f4out]));
% Plot OAL, BFL, TT, PTB, EFLRSS
% figure;plot(1:length(f1out),OALout,'k',1:length(f1out),BFLout,'b',1:length(f1out),TTout,'r');xlabel('Solution #');legend('OAL, L1-L4','BFL','TT, L1-IM')
figure;bar([OALout' BFLout' TTout']);xlabel('Solution #');legend('OAL, L_1-L_4','BFL','TT, L_1-IM')
% figure;plot(1:length(f1out),EFLRSS,'k');xlabel('Solution #');ylabel('RSS of Group Focal Lengths')
figure;bar(EFLRSS);xlabel('Solution #');ylabel('RSS of Group Focal Lengths')
% figure;plot(1:length(f1out),PTB,'k');xlabel('Solution #');ylabel('Sum of group power')
figure;bar(PTB);xlabel('Solution #');ylabel('Sum of group power')
figure('name','Minimum |EFL|');bar(minEFL);xlabel('Solution #');ylabel('minimum EFL magnitude')
figure;bar([f1out' f2out' f3out' f4out']);xlabel('Solution #');ylabel('Group Focal Length');legend('f_1','f_2','f_3','f_4')
end